@@ -22,9 +22,7 @@ from contextlib import contextmanager
 from pathlib import Path
 
 if T.TYPE_CHECKING:
-    # T.Union is not handled by flake8 to detect quoted annotation use (StringProtocol)
-    # see https://github.com/PyCQA/pyflakes/pull/632
-    from ._typing import StringProtocol, SizedStringProtocol # noqa: F401
+    from ._typing import StringProtocol, SizedStringProtocol
 
 """This is (mostly) a standalone module used to write logging
 information about Meson runs. Some output goes to screen,
@@ -256,7 +254,7 @@ def log(*args: TV_Loggable, is_error: bool = False,
 
 
 def _log(*args: TV_Loggable, is_error: bool = False,
-        **kwargs: T.Any) -> None:
+         **kwargs: T.Any) -> None:
     nested = kwargs.pop('nested', True)
     arr = process_markup(args, False)
     if log_file is not None:
